# Query Generation
MAX_NUM_QUERIES = 400
CHUNK_SIZE = 5
CHUNK_OVERLAP = 1
NUM_QUERIES_PER_PAGE = 1


# LLM call
LLM_MODEL = "gemini-2.5-flash"
API_KEY = 'AIzaSyD4TZ8PqRWTcs_8iDAHI-l8hAy4fQ5obVo'
MAX_WORKERS = 20
MAX_CONTEXT_LENGTH = 8000