if(!self.define){let e,i={};const n=(n,s)=>(n=new URL(n+".js",s).href,i[n]||new Promise(i=>{if("document"in self){const e=document.createElement("script");e.src=n,e.onload=i,document.head.appendChild(e)}else e=n,importScripts(n),i()}).then(()=>{let e=i[n];if(!e)throw new Error(`Module ${n} didn’t register its module`);return e}));self.define=(s,r)=>{const t=e||("document"in self?document.currentScript.src:"")||location.href;if(i[t])return;let o={};const c=e=>n(e,t),l={module:{uri:t},exports:o,require:c};i[t]=Promise.all(s.map(e=>l[e]||c(e))).then(e=>(r(...e),o))}}define(["./workbox-68854c78"],function(e){"use strict";self.addEventListener("message", e=>{e.data&&"SKIP_WAITING"===e.data.type&&self.skipWaiting()}),e.precacheAndRoute([{url:"/142.cbe331e0143c713c2b3a.js",revision:null},{url:"/index.html",revision:"4ac0b30be7bb6e4cb191fcb9e4eb5bc0"},{url:"/main.24f72c7eb5c45ebc6887.js",revision:null}],{})});
//# sourceMappingURL=service-worker.js.map
